# Select op files

Prior to `0.8.2` Guild selected all upstream run files by default when
it resolved an operation dependency. Post `0.8.2`, Guild only selects
upstream run-generated files.

Create a project with `up` and `down` operations.

    >>> use_project(mkdtemp())
    >>> write("guild.yml", """
    ... up:
    ...   requires:
    ...     - file: foo.txt
    ... down:
    ...   requires:
    ...     - operation: up
    ... """)

    >>> write("foo.txt", "Hello!\n")
    >>> write("up.py", "import shutil; shutil.copyfile('foo.txt', 'bar.txt')")
    >>> write("down.py", "print(open('bar.txt').read())")

Run `up`.

    >>> run("guild run up -y")
    Resolving file:foo.txt

The `up` operation requires `foo.txt` and generates `bar.txt`.

    >>> run("guild ls -nd")
    foo.txt

    >>> run("guild ls -ng")
    bar.txt

Run `down`.

    >>> run("guild run down -y")
    Resolving operation:up
    Using run ... for operation:up
    Hello!

Guild only selects the file generated by `up` as dependencies for `down`.

    >>> run("guild ls -nd")
    bar.txt

`down` does not generate any files.

    >>> run("guild ls -ng")
    <exit 0>

`foo.txt` in the case of `down` is copied as source code (default
rules).

    >>> run("guild ls -ns")
    down.py
    foo.txt
    guild.yml
    up.py

## Legacy support

Setting `GUILD_LEGACY_OPDEP_SELECT` to `1` causes Guild to select both
generated and dependencies.

    >>> with Env({"GUILD_LEGACY_OPDEP_SELECT": "1"}):
    ...     run("guild run down -y")
    Resolving operation:up
    Using run ... for operation:up
    WARNING: .../foo.txt already exists, skipping copy
    Hello!

Setting `GUILD_LEGACY_OPDEP_SELECT` to `2` causes Guild to select all
run files.

    >>> with Env({"GUILD_LEGACY_OPDEP_SELECT": "2"}):
    ...     run("guild run down -y")
    Resolving operation:up
    Using run ... for operation:up
    WARNING: .../down.py already exists, skipping copy
    WARNING: .../foo.txt already exists, skipping copy
    WARNING: .../guild.yml already exists, skipping copy
    WARNING: .../up.py already exists, skipping copy
    Hello!

Guild shows warnings for files that are already present in the `down`
run directory. These are copied as source code files.

    >>> run("guild ls -ns")
    down.py
    foo.txt
    guild.yml
    up.py

The only resolved upstream dependency is `bar.txt`.

    >>> run("guild ls -nd")
    bar.txt
